@@ -17,14 +17,13 @@
 import datetime
 import json
 import logging
-from enum import Enum
 from typing import List, Tuple, Any
 
 import certifi
 import urllib3
 from bs4 import BeautifulSoup
 
-from .provider import Suburb, Municipality, Provider, Province, ProviderError
+from .provider import Suburb, Municipality, Provider, Province, ProviderError, Stage
 
 
 class AreaInfo:
@@ -46,7 +45,7 @@ class Suburb(Suburb):
         self.id = kwargs.get("Id", kwargs.get("id"))
         self.name = kwargs.get("Name", kwargs.get("name"))
         self.municipality = Municipality(name=kwargs.get("MunicipalityName"))
-        self.province = Province(name=kwargs.get("ProvinceName"))
+        self.province = Eskom.province_from_name(kwargs.get("ProvinceName"))
         self.total = kwargs.get("Total")
         super().__init__(**kwargs)
 
@@ -61,75 +60,30 @@ class Municipality(Municipality):
         super().__init__(**kwargs)
 
 
-class Province(Province):
-    EASTERN_CAPE = 1
-    FREE_STATE = 2
-    GAUTENG_ = 3
-    KWAZULU_NATAL = 4
-    LIMPOPO_ = 6
-    MPUMALANGA = 5
-    NORTH_WEST = 7
-    NORTERN_CAPE = 8
-    WESTERN_CAPE = 9
-
-    def __init__(self, /, **kwargs):
-        self.id = kwargs.get("Id", kwargs.get("id"))
-        self.name = kwargs.get("Name", kwargs.get("name"))
-
-    def __str__(self):
-        return str(self.name)
-
-    def __repr__(self):
-        return self.id, self.name
-
-class Stage(Enum):
-    UNKNOWN = -1
-    NO_LOAD_SHEDDING = 0
-    STAGE_1 = 1
-    STAGE_2 = 2
-    STAGE_3 = 3
-    STAGE_4 = 4
-    STAGE_6 = 6
-    STAGE_5 = 5
-    STAGE_7 = 7
-    STAGE_8 = 8
-
-    @staticmethod
-    def from_status(status: str) -> Enum:
-        status_stage = {
-            "-1": Stage.NO_LOAD_SHEDDING.value,
-            "1": Stage.NO_LOAD_SHEDDING.value,
-            "2": Stage.STAGE_1.value,
-            "3": Stage.STAGE_2.value,
-            "4": Stage.STAGE_3.value,
-            "6": Stage.STAGE_4.value,
-            "5": Stage.STAGE_6.value,
-            "7": Stage.STAGE_5.value,
-            "8": Stage.STAGE_7.value,
-            "9": Stage.STAGE_8.value,
-        }
-        return Stage(status_stage.get(status, Stage.UNKNOWN))
-
-    def __str__(self):
-        stage_name = {
-            self.NO_LOAD_SHEDDING.value: "No Load Shedding",
-            self.STAGE_1.value: "Stage 1",
-            self.STAGE_2.value: "Stage 2",
-            self.STAGE_3.value: "Stage 3",
-            self.STAGE_4.value: "Stage 4",
-            self.STAGE_6.value: "Stage 6",
-            self.STAGE_5.value: "Stage 5",
-            self.STAGE_7.value: "Stage 7",
-            self.STAGE_8.value: "Stage 8",
-        }
-        return stage_name.get(self.value, "Unknown")
-
-
 # TODO: Improve error handling when Eskom responds with an error.
 class Eskom(Provider):
     def __init__(self):
         self.base_url = "https://loadshedding.eskom.co.za/LoadShedding"
 
+    @classmethod
+    def province_from_name(cls, name: str) -> Province:
+        return Provider.province_from_name(name)
+
+    @classmethod
+    def stage_from_status(cls, status: str) -> Stage:
+        return {
+            "-1": Stage.NO_LOAD_SHEDDING,
+            "1": Stage.NO_LOAD_SHEDDING,
+            "2": Stage.STAGE_1,
+            "3": Stage.STAGE_2,
+            "4": Stage.STAGE_3,
+            "6": Stage.STAGE_4,
+            "5": Stage.STAGE_6,
+            "7": Stage.STAGE_5,
+            "8": Stage.STAGE_7,
+            "9": Stage.STAGE_8,
+        }.get(status, Stage.UNKNOWN)
+
     def find_suburbs(self, search_text: str, max_results: int = 300) -> List[Suburb]:
         url = "{base_url}/FindSuburbs?searchText={search_text}&maxResults={max_results}".format(
             base_url=self.base_url,
@@ -142,23 +96,27 @@ class Eskom(Provider):
     def get_municipalities(self, province: Province) -> List[Municipality]:
         url = "{base_url}/GetMunicipalities/?Id={province}".format(
             base_url=self.base_url,
-            province=province.id,
+            province=province,
         )
         data = _get_response(url)
         return json.loads(data, object_hook=lambda d: Municipality(**d))
 
     def get_schedule(self, province: Province, suburb: Suburb, stage: Stage) -> Tuple[tuple]:
         schedule = []
-        url = "{base_url}/GetScheduleM/{suburb_id}/{stage}/{province_id}/3252".format(
+        url = "{base_url}/GetScheduleM/{suburb_id}/{stage}/{province}/3252".format(
             base_url=self.base_url,
             suburb_id=suburb.id,
-            province_id=province.id,
+            province=province.value,
             stage=stage.value,
         )
         data = _get_response(url)
         soup = BeautifulSoup(data, "html.parser")
         days_soup = soup.find_all("div", attrs={"class": "scheduleDay"})
 
+        if not days_soup:
+            logging.log(logging.ERROR, "Unable to parse schedule. {data}".format(data=data))
+            raise ProviderError("Eskom is unavailable.") from None
+
         now = datetime.datetime.now()
         for day in days_soup:
             date_soup = day.find("div", attrs={"class": "dayMonth"})
@@ -209,7 +167,7 @@ class Eskom(Provider):
     def get_stage(self) -> Stage:
         url = "{base_url}/GetStatus".format(base_url=self.base_url)
         data = _get_response(url)
-        return Stage.from_status(data.decode("utf-8"))
+        return Eskom.stage_from_status(data.decode("utf-8"))
 
 
 def _get_response(url: str) -> Any:
